@@ -42,6 +42,12 @@ SEXP xRMatrix_validate(SEXP x)
  * only with {32bit + R-devel + enable-R-shlib} -- no idea why */
 SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
 {
+    char *ncl = strdup(class_P(x));
+    static const char *valid[] = { MATRIX_VALID_Csparse, MATRIX_VALID_Rsparse, ""};
+    int ctype = R_check_class_etc(x, valid);
+    if (ctype < 0)
+	error(_("invalid class(x) '%s' in compressed_to_TMatrix(x)"), ncl);
+
     int col = asLogical(colP); /* 1 if "C"olumn compressed;  0 if "R"ow */
     /* however, for Csparse, we now effectively use the cholmod-based
      * Csparse_to_Tsparse() in ./Csparse.c ; maybe should simply write
@@ -50,12 +56,6 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
 	indP = PROTECT(GET_SLOT(x, indSym)),
 	  pP = PROTECT(GET_SLOT(x, Matrix_pSym));
     int npt = length(pP) - 1;
-    char *ncl = strdup(class_P(x));
-    static const char *valid[] = { MATRIX_VALID_Csparse, MATRIX_VALID_Rsparse, ""};
-    int ctype = R_check_class_etc(x, valid);
-
-    if (ctype < 0)
-	error(_("invalid class(x) '%s' in compressed_to_TMatrix(x)"), ncl);
 
     /* replace 'C' or 'R' with 'T' :*/
     ncl[2] = 'T';
@@ -82,16 +82,15 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
 
 SEXP R_to_CMatrix(SEXP x)
 {
-    SEXP ans, tri = PROTECT(allocVector(LGLSXP, 1));
     char *ncl = strdup(class_P(x));
     static const char *valid[] = { MATRIX_VALID_Rsparse, ""};
     int ctype = R_check_class_etc(x, valid);
-    int *x_dims = INTEGER(GET_SLOT(x, Matrix_DimSym)), *a_dims;
-    PROTECT_INDEX ipx;
-
     if (ctype < 0)
 	error(_("invalid class(x) '%s' in R_to_CMatrix(x)"), ncl);
 
+    SEXP ans, tri = PROTECT(allocVector(LGLSXP, 1));
+    int *x_dims = INTEGER(GET_SLOT(x, Matrix_DimSym)), *a_dims;
+    PROTECT_INDEX ipx;
     /* replace 'R' with 'C' : */
     ncl[2] = 'C';
     PROTECT_WITH_INDEX(ans = NEW_OBJECT_OF_CLASS(ncl), &ipx);
@@ -220,7 +219,7 @@ SEXP dgCMatrix_qrsol(SEXP x, SEXP y, SEXP ord)
 //  Usage: [V,beta,p,R,q] = cs_qr(A) ;
 SEXP dgCMatrix_QR(SEXP Ap, SEXP order, SEXP keep_dimnames)
 {
-    CSP A = AS_CSP__(Ap), D;
+    CSP A = AS_CSP__(Ap);
     int io = INTEGER(order)[0];
     Rboolean verbose = (io < 0);// verbose=TRUE, encoded with negative 'order'
     int m0 = A->m, m = m0, n = A->n, ord = asLogical(order) ? 3 : 0, *p;
@@ -239,10 +238,10 @@ SEXP dgCMatrix_QR(SEXP Ap, SEXP order, SEXP keep_dimnames)
     if(verbose && S->m2 > m) // in ./cs.h , m2 := # of rows for QR, after adding fictitious rows
 	Rprintf("Symbolic QR(): Matrix structurally rank deficient (m2-m = %d)\n",
 		S->m2 - m);
-    csn *N = cs_qr(A, S);		/* numeric QR factorization */
+    csn *N = cs_qr(A, S);	/* numeric QR factorization */
     if (!N) error(_("cs_qr failed")) ;
     cs_dropzeros(N->L);		/* drop zeros from V and sort */
-    D = cs_transpose(N->L, 1); cs_spfree(N->L);
+    CSP D = cs_transpose(N->L, 1); cs_spfree(N->L);
     N->L = cs_transpose(D, 1); cs_spfree(D);
     cs_dropzeros(N->U);		/* drop zeros from R and sort */
     D = cs_transpose(N->U, 1); cs_spfree(N->U) ;
@@ -284,7 +283,8 @@ SEXP dgCMatrix_QR(SEXP Ap, SEXP order, SEXP keep_dimnames)
 	}
     } else
 	ALLOC_SLOT(ans, install("q"), INTSXP, 0);
-    SET_SLOT(ans, install("R"), Matrix_cs_to_SEXP(N->U, "dgCMatrix", 0, dn));
+    SEXP R = PROTECT(Matrix_cs_to_SEXP(N->U, "dgCMatrix", 0, dn));
+    SET_SLOT(ans, Matrix_RSym, R); UNPROTECT(1); // R
     if(do_dn) UNPROTECT(1); // dn
     cs_nfree(N);
     cs_sfree(S);
@@ -338,15 +338,14 @@ SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
      * may make sense if to be used in the "spqr_solve" routines .. ?? */
 /*     SET_VECTOR_ELT(ans, 1, */
 /* 		   chm_sparse_to_SEXP(R, 0, 0, 0, "", R_NilValue)); */
-    SET_SLOT(ans, install("R"),
+    SET_SLOT(ans, Matrix_RSym,
 	     chm_sparse_to_SEXP(R, 0, 0, 0, "", R_NilValue));
     cholmod_free_sparse(&Al, &cl);
     cholmod_free_sparse(&R, &cl);
     cholmod_free_sparse(&Q, &cl);
     if (E) {
-	int *Er;
 	SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, A->ncol));
-	Er = INTEGER(VECTOR_ELT(ans, 2));
+	int *Er = INTEGER(VECTOR_ELT(ans, 2));
 	for (int i = 0; i < A->ncol; i++) Er[i] = (int) E[i];
 	Free(E);
     } else SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, 0));
@@ -361,23 +360,20 @@ SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
 void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep_dimnms)
 {
     // (order, tol) == (1, 1) by default, when called from R.
-    SEXP ans;
-    css *S;
-    csn *N;
-    int n, *p, *dims;
-    CSP A = AS_CSP__(Ap), D;
+    CSP A = AS_CSP__(Ap);
     R_CheckStack();
 
-    n = A->n;
+    int n = A->n;
     if (A->m != n)
 	error(_("LU decomposition applies only to square matrices"));
     if (order) {		/* not using natural order */
 	order = (tol == 1) ? 2	/* amd(S'*S) w/dense rows or I */
 	    : 1;		/* amd (A+A'), or natural */
     }
-    S = cs_sqr(order, A, /*qr = */ 0);	/* symbolic ordering */
-    N = cs_lu(A, S, tol);	/* numeric factorization */
+    css *S = cs_sqr(order, A, /*qr = */ 0);	/* symbolic ordering */
+    csn *N = cs_lu(A, S, tol);	/* numeric factorization */
     if (!N) {
+	cs_sfree(S);
 	if(err_sing)
 	    error(_("cs_lu(A) failed: near-singular A (or out of memory)"));
 	else {
@@ -388,7 +384,7 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep
 	}
     }
     cs_dropzeros(N->L);		/* drop zeros from L and sort it */
-    D = cs_transpose(N->L, 1);
+    CSP D = cs_transpose(N->L, 1);
     cs_spfree(N->L);
     N->L = cs_transpose(D, 1);
     cs_spfree(D);
@@ -397,9 +393,9 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep
     cs_spfree(N->U);
     N->U = cs_transpose(D, 1);
     cs_spfree(D);
-    p = cs_pinv(N->pinv, n);	/* p=pinv' */
-    ans = PROTECT(NEW_OBJECT_OF_CLASS("sparseLU"));
-    dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
+    int *p = cs_pinv(N->pinv, n);	/* p=pinv' */
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("sparseLU"));
+    int *dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = n; dims[1] = n;
     SEXP dn; Rboolean do_dn = FALSE;
     if(keep_dimnms) {
@@ -415,7 +411,7 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep
 	    SET_VECTOR_ELT(dn, 1, R_NilValue); // colnames(.) := NULL
 	}
     }
-    SET_SLOT(ans, install("L"),
+    SET_SLOT(ans, Matrix_LSym,
 	     Matrix_cs_to_SEXP(N->L, "dtCMatrix", 0, do_dn ? dn : R_NilValue));
 
     if(keep_dimnms) {
@@ -435,7 +431,7 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep
 	    SET_VECTOR_ELT(dn, 0, R_NilValue); // rownames(.) := NULL
 	}
     }
-    SET_SLOT(ans, install("U"),
+    SET_SLOT(ans, Matrix_USym,
 	     Matrix_cs_to_SEXP(N->U, "dtCMatrix", 0, do_dn ? dn : R_NilValue));
     if(do_dn) UNPROTECT(1); // dn
     Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_pSym, /* "p" */
@@ -501,24 +497,25 @@ SEXP dgCMatrix_matrix_solve(SEXP Ap, SEXP b, SEXP give_sparse)
 	lu = get_factors(Ap, "LU");
     }
     qslot = GET_SLOT(lu, install("q"));
-    L = AS_CSP__(GET_SLOT(lu, install("L")));
-    U = AS_CSP__(GET_SLOT(lu, install("U")));
+    L = AS_CSP__(GET_SLOT(lu, Matrix_LSym));
+    U = AS_CSP__(GET_SLOT(lu, Matrix_USym));
     R_CheckStack();
     if (U->n != n)
 	error(_("Dimensions of system to be solved are inconsistent"));
     if(nrhs >= 1 && n >= 1) {
+	R_xlen_t n_ = n; // <=> no overflow in j * n_
 	p = INTEGER(GET_SLOT(lu, Matrix_pSym));
 	q = LENGTH(qslot) ? INTEGER(qslot) : (int *) NULL;
 
 	for (j = 0; j < nrhs; j++) {
-	    cs_pvec(p, ax + j * n, x, n);  /* x = b(p) */
+	    cs_pvec(p, ax + j * n_, x, n);  /* x = b(p) */
 	    cs_lsolve(L, x);	       /* x = L\x */
 	    cs_usolve(U, x);	       /* x = U\x */
 	    if (q)		       /* r(q) = x , hence
 					  r = Q' U{^-1} L{^-1} P b = A^{-1} b */
-		cs_ipvec(q, x, ax + j * n, n);
+		cs_ipvec(q, x, ax + j * n_, n);
 	    else
-		Memcpy(ax + j * n, x, n);
+		Memcpy(ax + j * n_, x, n);
 	}
     }
     if(n >= SMALL_4_Alloca) Free(x);
@@ -535,7 +532,8 @@ SEXP dgCMatrix_cholsol(SEXP x, SEXP y)
      * cholmod_factorize("x", ..) finds L in  X'X = L'L directly */
     CHM_SP cx = AS_CHM_SP(x);
     /* FIXME: extend this to work in multivariate case, i.e. y a matrix with > 1 column ! */
-    CHM_DN cy = AS_CHM_DN(coerceVector(y, REALSXP)), rhs, cAns, resid;
+    SEXP y_ = PROTECT(coerceVector(y, REALSXP));
+    CHM_DN cy = AS_CHM_DN(y_), rhs, cAns, resid;
     CHM_FR L;
     int n = cx->ncol;/* #{obs.} {x = t(X) !} */
     double one[] = {1,0}, zero[] = {0,0}, neg1[] = {-1,0};
@@ -584,7 +582,7 @@ SEXP dgCMatrix_cholsol(SEXP x, SEXP y)
     cholmod_free_factor(&L, &c);
     cholmod_free_dense(&rhs, &c);
     cholmod_free_dense(&cAns, &c);
-    UNPROTECT(1);
+    UNPROTECT(2);
     return ans;
 }
 
